import pandas as pd
import os
import sys
import numpy as np
import dcarte
from dcarte.utils import process_transition, localize_time
from dcarte.local import LocalDataset
from dcarte.config import get_config
import dcarte


def map_devices(df, dt):
    dt.type = dt.type.str.replace('"|\[|\]', '',regex=True)\
                     .str.replace(' ', '_')
    mapping = dt.set_index('id').type.to_dict()
    df.device_type = df.device_type.replace(mapping).astype('category')
    return df


def process_physiology(self):
    _d1 = map_devices(self.datasets['vital_signs'],
                      self.datasets['device_types'])
    _d2 = map_devices(self.datasets['blood_pressure'],
                      self.datasets['device_types'])
    bp_types = np.array(['diastolic_value', 'systolic_value'])
    df = [_d1]

    for bp_type in bp_types:
        cols = _d2.columns[~_d2.columns.str.startswith(bp_type)]
        _tmp = _d2[cols].copy()
        _tmp.source = bp_types[bp_types != bp_type][0].replace('_value', '_bp')
        _tmp.columns = _d1.columns
        df.append(_tmp)
    df = pd.concat(df)
    dtypes = {'device_type': 'category', 'patient_id': 'category',
              'home_id': 'category', 'unit': 'category',
              'source': 'category', 'value': 'float'}
    df = df.astype(dtypes).reset_index(drop=True)
    df = localize_time(df,['start_date'])
    
    return df


def process_habitat(self):
    df = map_devices(self.datasets['environmental'],
                      self.datasets['device_types'])
    df = df[df.location_name != ''].reset_index(drop=True)
    dtypes = {'home_id': 'category', 'location_id': 'category',
              'unit': 'category', 'location_name': 'category',
              'source': 'category', 'value': 'float'}
    df = df.astype(dtypes)
    df = localize_time(df,['start_date'])
    return df


def process_doors(self):
    groupby = ['patient_id','location_name']
    datetime = 'start_date'
    value = 'value'
    covariates = ['location_name']
    df = self.datasets['door']
    df = localize_time(df,['start_date'])
    doors = process_transition(df,
                               groupby, 
                               datetime, 
                               value, 
                               covariates)
    return doors.drop('location_name',axis=1).reset_index()


def process_entryway(self):
    df = self.datasets['doors'].copy()
    df = df[df.location_name.isin(['front door', 'back door', 'main door'])]
    df.location_name = df.location_name.astype('str')\
                           .str.replace('main door', 'front door')\
                           .astype('category')
    df = df[df.transition == 'opened>closed']     

    return df.reset_index(drop=True)


def process_kitchen(self):
    # TODO: add sensor type as a column
    activity = self.datasets['activity']
    activity = localize_time(activity,['start_date'])
    doors = self.datasets['doors']
    appliances = map_devices(self.datasets['appliances'],
                             self.datasets['device_types'])
    doors = doors[(doors.location_name == 'fridge door') &
                  (doors.transition == 'opened>closed')]
    activity = activity[activity.location_name.isin(['kitchen',
                                                     'dining room'])]
    activity.insert(0,'device_type','pir')
    doors.insert(0,'device_type','door')
    appliances.value = appliances.value.astype(str)\
                                 .replace({'microwave-use': 'oven-use',
                                 'toaster-use': 'oven-use', 'iron-use': np.nan,
                                 'multi-use socket-use': np.nan})
    doors = doors[['patient_id', 'start_date', 'device_type','location_name']]\
                   .rename({'location_name': 'activity'}, axis=1)   
    activity = activity[['patient_id', 'start_date', 'device_type', 'location_name']]\
                        .rename({'location_name': 'activity'}, axis=1)       
    appliances = appliances[['patient_id', 'start_date', 'device_type', 'value']]\
                            .rename({'value': 'activity'}, axis=1) 
    kitchen = pd.concat([doors, activity, appliances.dropna()])   
    kitchen = kitchen[kitchen.patient_id != ''].sort_values(['patient_id', 'start_date']) 
    kitchen = process_transition(kitchen, ['patient_id'], 'start_date', 'activity', ['device_type'])
    return kitchen.reset_index(drop=True)


def process_motion(self):
    activity = self.datasets['activity']
    activity = localize_time(activity,['start_date'])
    entryway = self.datasets['entryway']
    bed_occupancy = self.datasets['bed_occupancy']
    fact = ['patient_id','location_name', 'start_date']
    motion = pd.concat([activity[fact], entryway[fact], bed_occupancy[fact]]).\
                        sort_values(['patient_id', 'start_date'])

    mapping = {'bathroom1': 'Bathroom', 
               'WC1': 'Bathroom',
               'kitchen': 'Kitchen',
               'hallway': 'Hallway',
               'corridor1': 'Hallway',
               'dining room': 'Lounge',
               'living room': 'Lounge',
               'lounge': 'Lounge',
               'bedroom1': 'Bedroom',
               'front door': 'Front door',
               'back door': 'Back door'}                    
    motion.location_name = motion.location_name.replace(mapping)

    motion = motion[~motion.location_name.isin(['office', 
                                                'conservatory', 
                                                'study', 
                                                'cellar'])]    
    return motion.reset_index(drop=True)

def process_transitions(self):
    motion = self.datasets['motion']
    motion = process_transition(motion, ['patient_id'], 'start_date', 'location_name')
    return motion.reset_index()


def mine_bed_occupancy(df):
    df = (df.
          assign(location_name=1).
          set_index('start_date').
          groupby('patient_id').
          resample('1T').mean())
    df = (df.
          location_name.
          fillna(0).
          diff().
          to_frame().
          query('location_name in [-1,1]').
          location_name.
          map({-1:'Bed_out',1:'Bed_in'}).
          to_frame())
    return df.reset_index()

def process_bed_occupancy(self):
    df = self.datasets['sleep']
    return  mine_bed_occupancy(df)

def process_sleep(self):
    """process_sleep force sleep_mat to 1min frequency and localise time 

    :return: loclaized sleep metrics timeseries 
    :rtype: pd.DataFrame
    """
    sleep_mat = self.datasets['sleep_mat']    
    sleep_mat = (sleep_mat.
                 set_index('start_date').
                 groupby(['patient_id','home_id']).
                 resample('1T').
                 agg({'snoring':'sum',
                      'heart_rate':'mean',
                      'respiratory_rate':'mean'}).
                 dropna().
                 reset_index())
    sleep_mat = localize_time(sleep_mat,['start_date'])
    return sleep_mat


def create_base_datasets():
    domain = 'base'
    module = 'base'
    # since = '2022-02-10'
    # until = '2022-02-20'
    parent_datasets = { 'Doors'     :[['door','raw']], 
                        'Entryway'  :[['doors','base']], 
                        'Habitat'   :[['environmental','raw'],
                                      ['device_types','lookup']], 
                        'Kitchen'   :[['appliances','raw'],
                                      ['doors','base'],
                                      ['activity','raw'],
                                      ['device_types','lookup']], 
                        'Motion'    :[['activity','raw'],
                                      ['entryway','base'],
                                      ['bed_occupancy','base']], 
                        'Physiology':[['vital_signs','raw'],
                                      ['blood_pressure','raw'],
                                      ['device_types','lookup']],
                        'Sleep'     :[['sleep_mat','raw']], 
                        'Bed_occupancy':[['sleep','base']], 
                        'Transitions':[['motion','base']]}
    
    module_path = __file__
    for dataset in ['Doors', 
                    'Entryway', 
                    'Habitat', 
                    'Kitchen', 
                    'Sleep',
                    'Bed_occupancy',
                    'Motion', 
                    'Physiology',
                    'Transitions']:
        
        p_datasets = {d[0]:dcarte.load(*d) for d in parent_datasets[dataset]} 
        
        LocalDataset(dataset_name = dataset,
                     datasets = p_datasets,
                     pipeline = [f'process_{dataset.lower()}'],
                     domain = domain,
                     module = module,
                     module_path = module_path,
                     reload = True,
                     dependencies = parent_datasets[dataset])
    
if __name__ == "__main__":
    create_base_datasets()  


import numpy as np 
import pandas as pd 
import matplotlib.pyplot as plt
import os
import sys
# This line allows you to clone the repository and skip installing dcarte
from scipy.stats import circmean,circstd
from dcarte.utils import (between_time,
                          time_to_angles)
import dcarte
from dcarte.local import LocalDataset

def mean_time(x): return pd.Series(circmean(x,high=360),name='mean')
def std_time(x): return pd.Series(circstd(x,high=360),name='std')


def process_activity_dailies(obj):
    """Activity_dailies creates a daily summary across key locations
    
    Args:
        obj ([LocalDataset]): [description]

    Returns:
        pd.DataFrame: a pandas dataframe with patient_id 
    """
    df = obj.datasets['motion']
    drop = ['bed_in','Back door','Front door']
    activity_metrics = (df.
        query('location_name not in @drop').
        assign(activity=True).
        set_index('start_date').
        groupby(['patient_id','location_name']).
        resample('1D',offset='12h').
        activity.
        count().
        swaplevel(-2,-1).
        unstack())
    activity_metrics['Total'] = activity_metrics.sum(axis=1)
    activity_metrics= activity_metrics.query('Total > 0.0')
    return activity_metrics

def process_activity_weeklies(obj):
    """activity_weeklies creates a weekly summary across key locations based on activity_dailies
    
    Args:
        obj ([LocalDataset]): [description]

    Returns:
        pd.DataFrame: a pandas dataframe with patient_id 
    """
    df = obj.datasets['activity_dailies']
    activity_weeklies = (df.
                        reset_index().
                        groupby('patient_id').
                        resample('1W',on='start_date').
                        agg({col:['mean','std'] for col in  df.columns}))
    return activity_weeklies

def process_sleep_dailies(obj):
    """sleep_dailies creates a daily summary across key features from the sleep mat
    
    Args:
        obj ([LocalDataset]): [description]

    Returns:
        pd.DataFrame: a pandas dataframe with patient_id 
    """
    df = obj.datasets['sleep']
    sleep_metrics = between_time(df,'start_date','17:00','11:00').copy()
    sleep_metrics['time'] = sleep_metrics['start_date']
    sleep_metrics['snoring'] = sleep_metrics['snoring'].astype(float)
    sleep_metrics = (sleep_metrics.set_index('start_date').
                        groupby('patient_id').
                        resample('1D',offset='12h').
                        agg({'heart_rate':['mean'],
                            'respiratory_rate':['mean'],
                            'snoring':['sum'],
                            'time':['first','last','count']}))
    sleep_metrics.columns = ['hr','br','snr','ttb','wup','tib']
    sleep_metrics['tib'] = sleep_metrics['tib']/60
    sleep_metrics['tob'] = (sleep_metrics.wup - sleep_metrics.ttb).dt.total_seconds()/60**2 - sleep_metrics.tib
    sleep_metrics.ttb = sleep_metrics.ttb.dt.time.apply(time_to_angles)
    sleep_metrics.wup = sleep_metrics.wup.dt.time.apply(time_to_angles)
    sleep_metrics.columns = ['Heart rate','Breathing rate','Snoring','Time to bed','Wake up time','Time in bed','Time out of bed']
    sleep_metrics = sleep_metrics.dropna(subset=['Heart rate','Breathing rate','Time to bed','Wake up time'])
    return sleep_metrics

def process_sleep_weeklies(obj):
    """sleep_weeklies creates a weekly summary across key features based on sleep_dailies
    
    Args:
        obj ([LocalDataset]): [description]

    Returns:
        pd.DataFrame: a pandas dataframe with patient_id 
    """
    df = obj.datasets['sleep_dailies']
    df_ = (df.
            reset_index().
            groupby('patient_id').
            resample('1W',on='start_date').
            agg({'Heart rate': ['mean','std'],
                'Breathing rate': ['mean','std'],
                'Snoring': ['mean','std'],
                'Time to bed': [mean_time,std_time],
                'Wake up time':   [mean_time,std_time],
                'Time in bed': ['mean','std'],
                'Time out of bed': ['mean','std']}))
    return df_

def process_physiology_dailies(obj):
    """physiology_dailies creates a daily summary across key features from the dialy vital signs
    
    Args:
        obj ([LocalDataset]): [description]

    Returns:
        pd.DataFrame: a pandas dataframe with patient_id 
    """
    df = obj.datasets['physiology']
    factors = ['raw_heart_rate','raw_body_weight','raw_body_mass_index',
            'raw_body_temperature','diastolic_bp','systolic_bp','raw_total_body_fat']
    daily_physiology = df.query("source in @factors")
    daily_physiology = (daily_physiology.reset_index(drop=True).
                            groupby(['patient_id','source']).
                            resample('1D',on='start_date',offset='12h').
                            agg({'value':'mean'}).
                            swaplevel(-2,-1).
                            unstack().
                            droplevel(0,axis=1))
    daily_physiology = daily_physiology[factors][daily_physiology[factors].isnull().sum(axis=1)<len(factors)]
    daily_physiology.columns = ['Heart rate','Body_weight','BMI','Temperature','Diastolic_BP','Systolic_BP', 'Body_Fat']
    
    return daily_physiology

def process_physiology_weeklies(obj):
    """physiology_weeklies creates a weekly summary across key features based on physiology_dailies
    
    Args:
        obj ([LocalDataset]): [description]

    Returns:
        pd.DataFrame: a pandas dataframe with patient_id 
    """
    df = obj.datasets['physiology_dailies']
    physiology_weeklies = (df.
                           reset_index().
                           groupby('patient_id').
                           resample('1W',on='start_date').
                           agg({col:['mean','std'] for col in  df.columns}))
    return physiology_weeklies

def process_light(obj):
    Habitat = obj.datasets['Habitat']
    Light = Habitat.query('source == "raw_light"')
    Light = (Light.groupby(['patient_id', 'location_name']).
                resample('15T', on='start_date').
                mean().fillna(method='ffill')).reset_index()
    Light.insert(3, 'date', Light.start_date.dt.date)
    Light.insert(4, 'time', Light.start_date.apply(time_to_angles))
    return Light

def process_temperature(obj):
    Habitat = obj.datasets['Habitat']
    temperature = Habitat.query('source != "raw_light"')
    temperature = (temperature.groupby(['patient_id', 'location_name']).
                resample('15T', on='start_date').
                mean().fillna(method='ffill')).reset_index()
    temperature.insert(3, 'date', temperature.start_date.dt.date)
    temperature.insert(4, 'time', temperature.start_date.apply(time_to_angles))
    return temperature

def create_weekly_profile():
    module_path = __file__
    # since = '2022-02-10'
    # until = '2022-02-20'
    module = "weekly_profile"
    domain = 'profile'
    parent_datasets = { 'activity_dailies':[['motion','base']], 
                        'activity_weeklies':[['activity_dailies','profile']], 
                        'sleep_dailies':[['sleep','base']], 
                        'sleep_weeklies':[['sleep_dailies','profile']], 
                        'physiology_dailies':[['physiology','base']], 
                        'physiology_weeklies':[['physiology_dailies','profile']],
                        'light':[['Habitat','base']], 
                        'temperature':[['Habitat','base']]}
    for dataset in parent_datasets.keys():
        p_datasets = {d[0]:dcarte.load(*d,
                                       since=since,
                                       until=until) for d in parent_datasets[dataset]} 
        LocalDataset(dataset_name = dataset,
                        datasets = p_datasets,
                        pipeline = [f'process_{dataset.lower()}'],
                        domain = domain,
                        module = module,
                        module_path = module_path,
                        reload = True,
                        dependencies = parent_datasets[dataset])
    
if __name__ == "__main__":
    create_weekly_profile()  


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dcarte', 'dcarte.derived', 'dcarte.legacy']

package_data = \
{'': ['*'], 'dcarte': ['source_yaml/*']}

install_requires = \
['PyYAML>=5.5',
 'deepdiff>=5.7.0,<6.0.0',
 'imbalanced-learn>=0.9.0,<0.10.0',
 'matplotlib>=3.5.1,<4.0.0',
 'numpy>=1.19.2',
 'pandas>=1.3.4',
 'pyarrow>=3.0.0,<6.0.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'requests>=2.26.0,<3.0.0',
 'scipy>=1.7.3,<2.0.0',
 'tqdm>=4.62.3,<5.0.0']

setup_kwargs = {
    'name': 'dcarte',
    'version': '0.3.25',
    'description': 'DCARTE is a dataset ingestion tool from DCARTE UK-DRI CAre Research and TEchnology',
    'long_description': None,
    'author': 'eyal soreq',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/esoreq/dcarte',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)
